#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.HasId import HasId
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.settings.Settings import Settings


class FunctionDefinition(HasId, SbmlObject):
	""" Function definition definition """

	def __init__ (self, model, obj_id):

		self.__model = model
		self.objId = obj_id

		HasId.__init__(self, model)
		SbmlObject.__init__(self, model)
		self.__definition = MathFormula(model, MathFormula.MATH_FUNCTION)

	def readSbml(self, sbml_function_definition, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Reads a function definition from a sbml file """

		HasId.readSbml(self, sbml_function_definition, sbml_level, sbml_version)
		SbmlObject.readSbml(self, sbml_function_definition, sbml_level, sbml_version)

		self.__definition.readSbml(sbml_function_definition.getMath(), sbml_level, sbml_version)

	def writeSbml(self, sbml_model, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes a function definition to a sbml file """

		t_function_definition = sbml_model.createFunctionDefinition()
		HasId.writeSbml(self, t_function_definition, sbml_level, sbml_version)
		SbmlObject.writeSbml(self, t_function_definition, sbml_level, sbml_version)
		t_function_definition.setMath(self.__definition.writeSbml(sbml_level, sbml_version))


	def copy(self, obj, sids_subs={}):
		HasId.copy(self, obj, sids_subs=sids_subs)
		SbmlObject.copy(self, obj)
		self.__definition.setInternalMathFormula(obj.getDefinition().getInternalMathFormula())

	def getDefinition(self):
		return self.__definition



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Initialization of the combine archive module

"""
from __future__ import absolute_import

from .CombineArchive import CombineArchive
from .CombineException import (
	FileNotFoundException, NotAZipFileException, NoManifestFoundException,
	NoMasterSedmlFoundException, NoMasterSbmlFoundException, NoMasterNumlFoundException,
	NoSedmlFoundException
)
__all__ = [
	'CombineArchive',
	'FileNotFoundException', 'NotAZipFileException', 'NoManifestFoundException',
	'NoMasterSedmlFoundException', 'NoMasterSbmlFoundException', 'NoMasterNumlFoundException',
	'NoSedmlFoundException'
]


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


class RuledVariable(object):


	def __init__(self, model):

		self.__model = model
		self.__isRuled = False
		self.__isRuledBy = None


	def setRuledBy(self, rule, shift=0):

		self.__isRuled = True
		self.__isRuledBy = rule.objId + shift

	def unsetRuledBy(self):

		self.__isRuled = False
		self.__isRuledById = None


	def isRuled(self):
		return self.__isRuled


	def isRuledBy(self):
		if self.isRuled():
			return self.__model.listOfRules[self.__isRuledBy]
		else:
			return None


	def isRateRuled(self):
		""" Tests is the compartment size is computed with a rate rule """
		return self.isRuled() and self.__model.listOfRules[self.__isRuledBy].isRate()


	def isAssignmentRuled(self):
		""" Tests is the compartment size is computed with a rate rule """
		return self.isRuled() and self.__model.listOfRules[self.__isRuledBy].isAssignment()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is made for 'high level' tests, using various components

"""

from libsignetsim import SbmlDocument, SedmlDocument, Settings
from unittest import TestCase
from os.path import join, dirname, isdir
from os import mkdir, getcwd


class TestMath(TestCase):
	""" Tests high level functions """

	def testMath(self):

		testfiles_path = join(dirname(join(getcwd(), __file__)), "files")

		if not isdir(testfiles_path):
			mkdir(testfiles_path)

		# Setting up the model
		sbml_doc = SbmlDocument()
		m = sbml_doc.model
		m.setName("Test")

		e = m.listOfSpecies.new("E", value=10)
		s = m.listOfSpecies.new("S", value=12)
		p = m.listOfSpecies.new("P", value=0)

		sbml_filename = join(Settings.tempDirectory, "testMath.sbml")
		sbml_doc.writeSbmlToFile(sbml_filename)

		sedml_doc = SedmlDocument()

		simulation = sedml_doc.listOfSimulations.createUniformTimeCourse()
		simulation.setInitialTime(0)
		simulation.setOutputStartTime(0)
		simulation.setOutputEndTime(10)
		simulation.setNumberOfPoints(100)
		simulation.getAlgorithm().setCVODE()
		simulation.getAlgorithm().listOfAlgorithmParameters.setRelTol(1e-6)
		simulation.getAlgorithm().listOfAlgorithmParameters.setAbsTol(1e-30)

		model = sedml_doc.listOfModels.createModel()
		model.setLanguageSbml()
		model.setSource(sbml_filename)

		task = sedml_doc.listOfTasks.createTask()
		task.setModel(model)
		task.setSimulation(simulation)

		# >> pi*time
		data_0 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_0.setName("data_0")
		time_0 = data_0.listOfVariables.createVariable("time")
		time_0.setTask(task)
		time_0.setModel(model)
		time_0.setSymbolTime()
		data_0.getMath().setPrettyPrintMathFormula("pi*time")

		# >> piecewise(2, time > 0.5, 3)
		data_1 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_1.setName("data_1")
		time_1 = data_1.listOfVariables.createVariable("time")
		time_1.setTask(task)
		time_1.setModel(model)
		time_1.setSymbolTime()
		data_1.getMath().setPrettyPrintMathFormula("piecewise(2, time > 0.5, 3)")

		# >> piecewise(2, time >= 0.5, 3)
		data_2 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_2.setName("data_2")
		time_2 = data_2.listOfVariables.createVariable("time")
		time_2.setTask(task)
		time_2.setModel(model)
		time_2.setSymbolTime()
		data_2.getMath().setPrettyPrintMathFormula("piecewise(2, time >= 0.5, 3)")

		# >> piecewise(2, time < 0.5, 3)
		data_3 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_3.setName("data_3")
		time_3 = data_3.listOfVariables.createVariable("time")
		time_3.setTask(task)
		time_3.setModel(model)
		time_3.setSymbolTime()
		data_3.getMath().setPrettyPrintMathFormula("piecewise(2, time < 0.5, 3)")

		# >> piecewise(2, time <= 0.5, 3)
		data_4 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_4.setName("data_4")
		time_4 = data_4.listOfVariables.createVariable("time")
		time_4.setTask(task)
		time_4.setModel(model)
		time_4.setSymbolTime()
		data_4.getMath().setPrettyPrintMathFormula("piecewise(2, time <= 0.5, 3)")

		# >> piecewise(2, time == pi, 3)
		data_5 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_5.setName("data_5")
		time_5 = data_5.listOfVariables.createVariable("time")
		time_5.setTask(task)
		time_5.setModel(model)
		time_5.setSymbolTime()
		data_5.getMath().setPrettyPrintMathFormula("piecewise(2, time == pi, 3)")

		# >> piecewise(2, P1 != pi, 3)
		data_6 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_6.setName("data_6")
		param_6 = data_6.listOfParameters.createParameter("p1")
		param_6.setValue(2)
		time_6 = data_6.listOfVariables.createVariable("time")
		time_6.setTask(task)
		time_6.setModel(model)
		time_6.setSymbolTime()
		data_6.getMath().setPrettyPrintMathFormula("piecewise(2, p1 != time, 3)")

		# >> factorial(4)
		data_7 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_7.setName("data_7")
		time_7 = data_7.listOfVariables.createVariable("time")
		time_7.setTask(task)
		time_7.setModel(model)
		time_7.setSymbolTime()
		data_7.getMath().setPrettyPrintMathFormula("factorial(4)")

		# >> piecewise(3, true && false, 3)
		data_8 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_8.setName("data_8")
		time_8 = data_8.listOfVariables.createVariable("time")
		time_8.setTask(task)
		time_8.setModel(model)
		time_8.setSymbolTime()
		data_8.getMath().setPrettyPrintMathFormula("piecewise(3, (time < 2) && false, time)")

		# >> piecewise(2, true || false, 3)
		data_9 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_9.setName("data_9")
		time_9 = data_9.listOfVariables.createVariable("time")
		time_9.setTask(task)
		time_9.setModel(model)
		time_9.setSymbolTime()
		data_9.getMath().setPrettyPrintMathFormula("piecewise(2, true || (time >= 2), time)")

		# >> piecewise(2, xor(true, false), 3)
		data_10 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_10.setName("data_10")
		time_10 = data_10.listOfVariables.createVariable("time")
		time_10.setTask(task)
		time_10.setModel(model)
		time_10.setSymbolTime()
		data_10.getMath().setPrettyPrintMathFormula("piecewise(2, xor(true, time == 2), time)")

		# >> piecewise(2, !false, 3)
		data_11 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_11.setName("data_11")
		time_11 = data_11.listOfVariables.createVariable("time")
		time_11.setTask(task)
		time_11.setModel(model)
		time_11.setSymbolTime()
		data_11.getMath().setPrettyPrintMathFormula("piecewise(2, !(time != 2), time)")

		# >> sec(0.5)
		data_12 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_12.setName("data_12")
		time_12 = data_12.listOfVariables.createVariable("time")
		time_12.setTask(task)
		time_12.setModel(model)
		time_12.setSymbolTime()
		data_12.getMath().setPrettyPrintMathFormula("sec(time)")

		# >> csc(4.5)
		data_13 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_13.setName("data_13")
		time_13 = data_13.listOfVariables.createVariable("time")
		time_13.setTask(task)
		time_13.setModel(model)
		time_13.setSymbolTime()
		data_13.getMath().setPrettyPrintMathFormula("csc(time)")

		# >> cot(0.2)
		data_14 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_14.setName("data_14")
		time_14 = data_14.listOfVariables.createVariable("time")
		time_14.setTask(task)
		time_14.setModel(model)
		time_14.setSymbolTime()
		data_14.getMath().setPrettyPrintMathFormula("cot(time)")

		# >> sinh(0.3)
		data_15 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_15.setName("data_15")
		time_15 = data_15.listOfVariables.createVariable("time")
		time_15.setTask(task)
		time_15.setModel(model)
		time_15.setSymbolTime()
		data_15.getMath().setPrettyPrintMathFormula("sinh(time)")

		# >> cosh(1.7)
		data_16 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_16.setName("data_16")
		time_16 = data_16.listOfVariables.createVariable("time")
		time_16.setTask(task)
		time_16.setModel(model)
		time_16.setSymbolTime()
		data_16.getMath().setPrettyPrintMathFormula("cosh(time)")

		# >> arcsec(2.3)
		data_17 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_17.setName("data_17")
		time_17 = data_17.listOfVariables.createVariable("time")
		time_17.setTask(task)
		time_17.setModel(model)
		time_17.setSymbolTime()
		data_17.getMath().setPrettyPrintMathFormula("arcsec(time)")

		# >> arccsc(1.1)
		data_18 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_18.setName("data_18")
		time_18 = data_18.listOfVariables.createVariable("time")
		time_18.setTask(task)
		time_18.setModel(model)
		time_18.setSymbolTime()
		data_18.getMath().setPrettyPrintMathFormula("arccsc(time+1)")

		# >> arccot(-0.1)
		data_19 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_19.setName("data_19")
		time_19 = data_19.listOfVariables.createVariable("time")
		time_19.setTask(task)
		time_19.setModel(model)
		time_19.setSymbolTime()
		data_19.getMath().setPrettyPrintMathFormula("arccot(time+1)")

		# >> arcsinh(99)
		data_20 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_20.setName("data_20")
		time_20 = data_20.listOfVariables.createVariable("time")
		time_20.setTask(task)
		time_20.setModel(model)
		time_20.setSymbolTime()
		data_20.getMath().setPrettyPrintMathFormula("arcsinh(time)")

		# >> arccosh(1.34)
		data_21 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_21.setName("data_21")
		time_21 = data_21.listOfVariables.createVariable("time")
		time_21.setTask(task)
		time_21.setModel(model)
		time_21.setSymbolTime()
		data_21.getMath().setPrettyPrintMathFormula("arccosh(time)")

		# >> arctanh(-0.7)
		data_22 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_22.setName("data_22")
		time_22 = data_22.listOfVariables.createVariable("time")
		time_22.setTask(task)
		time_22.setModel(model)
		time_22.setSymbolTime()
		data_22.getMath().setPrettyPrintMathFormula("arctanh(time)")

		# >> arcsech(0.42)
		data_23 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_23.setName("data_23")
		time_23 = data_23.listOfVariables.createVariable("time")
		time_23.setTask(task)
		time_23.setModel(model)
		time_23.setSymbolTime()
		data_23.getMath().setPrettyPrintMathFormula("arcsech(time+1)")

		# >> arccsch(0.01)
		data_24 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_24.setName("data_24")
		time_24 = data_24.listOfVariables.createVariable("time")
		time_24.setTask(task)
		time_24.setModel(model)
		time_24.setSymbolTime()
		data_24.getMath().setPrettyPrintMathFormula("arccsc(time+1)")

		# >> arccoth(8.2)
		data_25 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_25.setName("data_25")
		time_25 = data_25.listOfVariables.createVariable("time")
		time_25.setTask(task)
		time_25.setModel(model)
		time_25.setSymbolTime()
		data_25.getMath().setPrettyPrintMathFormula("arccoth(time+1)")

		# >> exponentiale
		data_26 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_26.setName("data_26")
		time_26 = data_26.listOfVariables.createVariable("time")
		time_26.setTask(task)
		time_26.setModel(model)
		time_26.setSymbolTime()
		data_26.getMath().setPrettyPrintMathFormula("exponentiale")

		# >> exp(exponentiale)
		data_27 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_27.setName("data_27")
		time_27 = data_27.listOfVariables.createVariable("time")
		time_27.setTask(task)
		time_27.setModel(model)
		time_27.setSymbolTime()
		data_27.getMath().setPrettyPrintMathFormula("exp(exponentiale)")

		# >> 3.7
		data_28 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_28.setName("data_28")
		time_28 = data_28.listOfVariables.createVariable("time")
		time_28.setTask(task)
		time_28.setModel(model)
		time_28.setSymbolTime()
		data_28.getMath().setPrettyPrintMathFormula("3.7")

		# >> abs(-1)
		data_29 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_29.setName("data_29")
		time_29 = data_29.listOfVariables.createVariable("time")
		time_29.setTask(task)
		time_29.setModel(model)
		time_29.setSymbolTime()
		data_29.getMath().setPrettyPrintMathFormula("abs(-time)")

		# >> abs(1)
		data_30 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_30.setName("data_30")
		time_30 = data_30.listOfVariables.createVariable("time")
		time_30.setTask(task)
		time_30.setModel(model)
		time_30.setSymbolTime()
		data_30.getMath().setPrettyPrintMathFormula("abs(time)")

		# >> acos(-1)
		data_31 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_31.setName("data_31")
		time_31 = data_31.listOfVariables.createVariable("time")
		time_31.setTask(task)
		time_31.setModel(model)
		time_31.setSymbolTime()
		data_31.getMath().setPrettyPrintMathFormula("acos(time)")

		# >> acos(0.5)
		data_32 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_32.setName("data_32")
		time_32 = data_32.listOfVariables.createVariable("time")
		time_32.setTask(task)
		time_32.setModel(model)
		time_32.setSymbolTime()
		data_32.getMath().setPrettyPrintMathFormula("arccos(time)")

		# >> asin(1)
		data_33 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_33.setName("data_33")
		time_33 = data_33.listOfVariables.createVariable("time")
		time_33.setTask(task)
		time_33.setModel(model)
		time_33.setSymbolTime()
		data_33.getMath().setPrettyPrintMathFormula("arcsin(time)")

		# >> asin(-0.5)
		data_34 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_34.setName("data_34")
		time_34 = data_34.listOfVariables.createVariable("time")
		time_34.setTask(task)
		time_34.setModel(model)
		time_34.setSymbolTime()
		data_34.getMath().setPrettyPrintMathFormula("arcsin(time)")

		# >> atan(2.8)
		data_35 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_35.setName("data_35")
		time_35 = data_35.listOfVariables.createVariable("time")
		time_35.setTask(task)
		time_35.setModel(model)
		time_35.setSymbolTime()
		data_35.getMath().setPrettyPrintMathFormula("arctan(time)")

		# >> atan(-7.09)
		data_36 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_36.setName("data_36")
		time_36 = data_36.listOfVariables.createVariable("time")
		time_36.setTask(task)
		time_36.setModel(model)
		time_36.setSymbolTime()
		data_36.getMath().setPrettyPrintMathFormula("arctan(time)")

		# >> ceil(0.5)
		data_37 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_37.setName("data_37")
		time_37 = data_37.listOfVariables.createVariable("time")
		time_37.setTask(task)
		time_37.setModel(model)
		time_37.setSymbolTime()
		data_37.getMath().setPrettyPrintMathFormula("ceil(time)")

		# >> ceil(3.55)
		data_38 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_38.setName("data_38")
		time_38 = data_38.listOfVariables.createVariable("time")
		time_38.setTask(task)
		time_38.setModel(model)
		time_38.setSymbolTime()
		data_38.getMath().setPrettyPrintMathFormula("ceil(time+0.05)")

		# >> ceil(-4.6)
		data_39 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_39.setName("data_39")
		time_39 = data_39.listOfVariables.createVariable("time")
		time_39.setTask(task)
		time_39.setModel(model)
		time_39.setSymbolTime()
		data_39.getMath().setPrettyPrintMathFormula("ceil(-time)")

		# >> cos(9.1)
		data_40 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_40.setName("data_40")
		time_40 = data_40.listOfVariables.createVariable("time")
		time_40.setTask(task)
		time_40.setModel(model)
		time_40.setSymbolTime()
		data_40.getMath().setPrettyPrintMathFormula("cos(time)")

		# >> cos(-0.22)
		data_41 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_41.setName("data_41")
		time_41 = data_41.listOfVariables.createVariable("time")
		time_41.setTask(task)
		time_41.setModel(model)
		time_41.setSymbolTime()
		data_41.getMath().setPrettyPrintMathFormula("cos(-time)")

		# >> exp(0)
		data_42 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_42.setName("data_42")
		time_42 = data_42.listOfVariables.createVariable("time")
		time_42.setTask(task)
		time_42.setModel(model)
		time_42.setSymbolTime()
		data_42.getMath().setPrettyPrintMathFormula("exp(time+1)")

		# >> exp(1)
		data_43 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_43.setName("data_43")
		time_43 = data_43.listOfVariables.createVariable("time")
		time_43.setTask(task)
		time_43.setModel(model)
		time_43.setSymbolTime()
		data_43.getMath().setPrettyPrintMathFormula("exp(time)")

		# >> exp(0.77)
		data_44 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_44.setName("data_44")
		time_44 = data_44.listOfVariables.createVariable("time")
		time_44.setTask(task)
		time_44.setModel(model)
		time_44.setSymbolTime()
		data_44.getMath().setPrettyPrintMathFormula("exp(0.77)")

		# >> floor(-4.6)
		data_45 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_45.setName("data_45")
		time_45 = data_45.listOfVariables.createVariable("time")
		time_45.setTask(task)
		time_45.setModel(model)
		time_45.setSymbolTime()
		data_45.getMath().setPrettyPrintMathFormula("floor(-time)")

		# >> floor(9.1)
		data_46 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_46.setName("data_46")
		time_46 = data_46.listOfVariables.createVariable("time")
		time_46.setTask(task)
		time_46.setModel(model)
		time_46.setSymbolTime()
		data_46.getMath().setPrettyPrintMathFormula("floor(time)")

		# >> ln(0.2)
		data_47 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_47.setName("data_47")
		time_47 = data_47.listOfVariables.createVariable("time")
		time_47.setTask(task)
		time_47.setModel(model)
		time_47.setSymbolTime()
		data_47.getMath().setPrettyPrintMathFormula("ln(time+1e-6)")

		# >> ln(1)
		data_48 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_48.setName("data_48")
		time_48 = data_48.listOfVariables.createVariable("time")
		time_48.setTask(task)
		time_48.setModel(model)
		time_48.setSymbolTime()
		data_48.getMath().setPrettyPrintMathFormula("ln(1)")

		# >> log10(0.2)
		data_49 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_49.setName("data_49")
		time_49 = data_49.listOfVariables.createVariable("time")
		time_49.setTask(task)
		time_49.setModel(model)
		time_49.setSymbolTime()
		data_49.getMath().setPrettyPrintMathFormula("log10(time+0.2)")

		# >> log10(1)
		data_50 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_50.setName("data_50")
		time_50 = data_50.listOfVariables.createVariable("time")
		time_50.setTask(task)
		time_50.setModel(model)
		time_50.setSymbolTime()
		data_50.getMath().setPrettyPrintMathFormula("log10(1.0)")

		# >> 1^2
		data_51 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_51.setName("data_51")
		time_51 = data_51.listOfVariables.createVariable("time")
		time_51.setTask(task)
		time_51.setModel(model)
		time_51.setSymbolTime()
		data_51.getMath().setPrettyPrintMathFormula("1^2")

		# >> 2^2
		data_52 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_52.setName("data_52")
		time_52 = data_52.listOfVariables.createVariable("time")
		time_52.setTask(task)
		time_52.setModel(model)
		time_52.setSymbolTime()
		data_52.getMath().setPrettyPrintMathFormula("time^2")

		# >> 1.4^5.1
		data_53 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_53.setName("data_53")
		time_53 = data_53.listOfVariables.createVariable("time")
		time_53.setTask(task)
		time_53.setModel(model)
		time_53.setSymbolTime()
		data_53.getMath().setPrettyPrintMathFormula("1.4^5.1")

		# >> 4^2
		data_54 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_54.setName("data_54")
		time_54 = data_54.listOfVariables.createVariable("time")
		time_54.setTask(task)
		time_54.setModel(model)
		time_54.setSymbolTime()
		data_54.getMath().setPrettyPrintMathFormula("4^time")

		# >> 3.1^2
		data_55 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_55.setName("data_55")
		time_55 = data_55.listOfVariables.createVariable("time")
		time_55.setTask(task)
		time_55.setModel(model)
		time_55.setSymbolTime()
		data_55.getMath().setPrettyPrintMathFormula("3.1^2")

		# >> sqrt(4)
		data_56 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_56.setName("data_56")
		time_56 = data_56.listOfVariables.createVariable("time")
		time_56.setTask(task)
		time_56.setModel(model)
		time_56.setSymbolTime()
		data_56.getMath().setPrettyPrintMathFormula("sqrt(time)")

		# >> sqrt(7.4)
		data_57 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_57.setName("data_57")
		time_57 = data_57.listOfVariables.createVariable("time")
		time_57.setTask(task)
		time_57.setModel(model)
		time_57.setSymbolTime()
		data_57.getMath().setPrettyPrintMathFormula("sqrt(time+2)")

		# >> sin(2.1)
		data_58 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_58.setName("data_58")
		time_58 = data_58.listOfVariables.createVariable("time")
		time_58.setTask(task)
		time_58.setModel(model)
		time_58.setSymbolTime()
		data_58.getMath().setPrettyPrintMathFormula("sin(time)")

		# >> sin(0)
		data_59 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_59.setName("data_59")
		time_59 = data_59.listOfVariables.createVariable("time")
		time_59.setTask(task)
		time_59.setModel(model)
		time_59.setSymbolTime()
		data_59.getMath().setPrettyPrintMathFormula("sin(0)")

		# >> sin(-5.9)
		data_60 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_60.setName("data_60")
		time_60 = data_60.listOfVariables.createVariable("time")
		time_60.setTask(task)
		time_60.setModel(model)
		time_60.setSymbolTime()
		data_60.getMath().setPrettyPrintMathFormula("sin(-time)")

		# >> tan(0)
		data_61 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_61.setName("data_61")
		time_61 = data_61.listOfVariables.createVariable("time")
		time_61.setTask(task)
		time_61.setModel(model)
		time_61.setSymbolTime()
		data_61.getMath().setPrettyPrintMathFormula("tan(time)")

		# >> tan(1.11)
		data_62 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_62.setName("data_62")
		time_62 = data_62.listOfVariables.createVariable("time")
		time_62.setTask(task)
		time_62.setModel(model)
		time_62.setSymbolTime()
		data_62.getMath().setPrettyPrintMathFormula("tan(time+1.11)")

		# >> tan(-6)
		data_63 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_63.setName("data_63")
		time_63 = data_63.listOfVariables.createVariable("time")
		time_63.setTask(task)
		time_63.setModel(model)
		time_63.setSymbolTime()
		data_63.getMath().setPrettyPrintMathFormula("tan(-6*time)")

		# >> 1 + 2
		data_64 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_64.setName("data_64")
		time_64 = data_64.listOfVariables.createVariable("time")
		time_64.setTask(task)
		time_64.setModel(model)
		time_64.setSymbolTime()
		data_64.getMath().setPrettyPrintMathFormula("1+2")

		# >> 1 + -2
		data_65 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_65.setName("data_65")
		time_65 = data_65.listOfVariables.createVariable("time")
		time_65.setTask(task)
		time_65.setModel(model)
		time_65.setSymbolTime()
		data_65.getMath().setPrettyPrintMathFormula("1 + -2")

		# # >> (5/2)
		# data_66 = sedml_doc.listOfDataGenerators.createDataGenerator()
		# data_66.setName("data_66")
		# time_66 = data_66.listOfVariables.createVariable("time")
		# time_66.setTask(task)
		# time_66.setModel(model)
		# time_66.setSymbolTime()
		# data_66.getMath().setPrettyPrintMathFormula("(5/2)")

		# >> 8 * 3
		data_67 = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_67.setName("data_67")
		time_67 = data_67.listOfVariables.createVariable("time")
		time_67.setTask(task)
		time_67.setModel(model)
		time_67.setSymbolTime()
		data_67.getMath().setPrettyPrintMathFormula("8.3")

		# # >> lambda(time, p1, time*p1)
		# data_68 = sedml_doc.listOfDataGenerators.createDataGenerator()
		# data_68.setName("data_50")
		# param_68 = data_68.listOfParameters.createParameter("p1")
		# param_68.setValue(2.0)
		# time_68 = data_68.listOfVariables.createVariable("time")
		# time_68.setTask(task)
		# time_68.setModel(model)
		# time_68.setSymbolTime()
		# data_68.getMath().setPrettyPrintMathFormula("lambda(time, p1, (time+1)*p1)")

		report = sedml_doc.listOfOutputs.createReport()
		dataset_0 = report.listOfDataSets.createDataSet()
		dataset_0.setLabel("data_0")
		dataset_0.setData(data_0)


		sedml_doc.run()
		sedml_filename = join(Settings.tempDirectory, "testMath.sedml")
		sedml_doc.writeSedmlToFile(sedml_filename)

		sedml_doc_2 = SedmlDocument()
		sedml_doc_2.readSedmlFromFile(sedml_filename)
		sedml_doc.run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.UniformRange import UniformRange
from libsignetsim.sedml.VectorRange import VectorRange
from libsignetsim.sedml.FunctionalRange import FunctionalRange


from libsignetsim.settings.Settings import Settings

import libsbml
from libsedml import SEDML_RANGE_UNIFORMRANGE, SEDML_RANGE_VECTORRANGE, SEDML_RANGE_FUNCTIONALRANGE
from six.moves import reload_module
reload_module(libsbml)


class ListOfRanges(ListOf):

	def __init__(self, document, repeated_task):

		ListOf.__init__(self, document)
		self.__document = document
		self.__repeatedTask = repeated_task
		self.__rangeCounter = 0

	def new(self, range_type, range_id=None):

		t_range_id = range_id
		if t_range_id is None:
			t_range_id = "%s_range_%d" % (self.__repeatedTask.getId(), self.__rangeCounter)

		if range_type == SEDML_RANGE_UNIFORMRANGE:
			t_range = UniformRange(self.__document)
			t_range.setId(t_range_id)
			ListOf.append(self, t_range)
			self.__rangeCounter += 1
			return t_range

		elif range_type == SEDML_RANGE_VECTORRANGE:
			t_range = VectorRange(self.__document)
			t_range.setId(t_range_id)
			ListOf.append(self, t_range)
			self.__rangeCounter += 1
			return t_range

		elif range_type == SEDML_RANGE_FUNCTIONALRANGE:
			t_range = FunctionalRange(self.__document)
			t_range.setId(t_range_id)
			ListOf.append(self, t_range)
			self.__rangeCounter += 1
			return t_range

	def createUniformRange(self, range_id=None):
		return self.new(SEDML_RANGE_UNIFORMRANGE, range_id)

	def createVectorRange(self, range_id=None):
		return self.new(SEDML_RANGE_VECTORRANGE, range_id)

	def createFunctionalRange(self, range_id=None):
		return self.new(SEDML_RANGE_FUNCTIONALRANGE, range_id)

	def readSedml(self, list_of_ranges, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_ranges, level, version)

		for tt_range in list_of_ranges:

			if tt_range.getTypeCode() == SEDML_RANGE_UNIFORMRANGE:
				t_range = UniformRange(self.__document)
				t_range.readSedml(tt_range, level, version)
				ListOf.append(self, t_range)
				self.__rangeCounter += 1

			elif tt_range.getTypeCode() == SEDML_RANGE_VECTORRANGE:
				t_range = VectorRange(self.__document)
				t_range.readSedml(tt_range, level, version)
				ListOf.append(self, t_range)
				self.__rangeCounter += 1

			elif tt_range.getTypeCode() == SEDML_RANGE_FUNCTIONALRANGE:
				t_range = FunctionalRange(self.__document)
				t_range.readSedml(tt_range, level, version)
				ListOf.append(self, t_range)
				self.__rangeCounter += 1

	def writeSedml(self, list_of_ranges, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_ranges, level, version)

		for tt_range in self:

			if isinstance(tt_range, UniformRange):
				t_range = list_of_ranges.createUniformRange()
				tt_range.writeSedml(t_range, level, version)

			elif isinstance(tt_range, VectorRange):
				t_range = list_of_ranges.createVectorRange()
				tt_range.writeSedml(t_range, level, version)

			elif isinstance(tt_range, FunctionalRange):
				t_range = list_of_ranges.createFunctionalRange()
				tt_range.writeSedml(t_range, level, version)

	def getByRangeId(self, range_id):

		for t_range in self:
			if t_range.getId() == range_id:
				return t_range



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Retryiving Pubmed URI informations

"""


import requests
from json import loads

class PubmedResolver(object):

	URL = "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?db=pubmed&retmode=json&id=%s"
	URL_ABSTRACTS = "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&retmode=text&rettype=abstract&id=%s"

	def __init__(self, pubmed_id):
		self.__id = pubmed_id
		self.__json = None
		self.__text_abstract = None
		self.__name = None
		self.__definition = None

	def lookup(self):

		if self.__id is not None:
			# self.getJSON()
			# self.parseJSON()
			self.getRawAbstract()
			self.parseRawAbstract()

	def getJSON(self):

		r = requests.get(self.URL % self.__id, headers={"Content-Type": "application/json"})

		if r.ok:
			self.__json = loads(r.content.decode('utf-8'))
			print(self.__json)
			self.__json = self.__json[u'result'][str(self.__id)]

	def parseJSON(self):

		if self.__json is not None:

			self.__name = (self.__json[u'sortfirstauthor']
						   + ', '
						   + self.__json[u'sorttitle']
						   + ", "
						   + self.__json[u'fulljournalname']
						   + ", "
						   + self.__json[u'pubdate'])

			self.__definition = self.__json[u'sortfirstauthor']

	def getRawAbstract(self):

		r = requests.get(self.URL_ABSTRACTS % self.__id, headers={"Content-Type": "application/json"})

		if r.ok:
			self.__text_abstract = r.content.decode('utf-8')

	def parseRawAbstract(self):

		lines = self.__text_abstract.strip().split("\n\n")
		self.__name = (
			lines[2].replace('\n', ' ').replace('\r', '')
			+ ", " + lines[1].replace('\n', ' ').replace('\r', '')
			+ ", " + lines[0].replace('\n', ' ').replace('\r', '')
		)
		self.__definition = lines[-2].replace('\n', ' ').replace('\r', '')

	def getName(self):
		return self.__name

	def getDefinition(self):
		return self.__definition


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.ModelException import InvalidXPath
from libsignetsim.model.sbml.Event import Event
from libsignetsim.settings.Settings import Settings

from re import match

class ListOfEvents(ListOf, HasIds, SbmlObject, HasParentObj):
	""" Class for the listOfEvents in a sbml model """

	def __init__(self, model, parent_obj, math_only=False):

		self.__model = model
		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		HasParentObj.__init__(self, parent_obj)

		# For math submodels, where objects are not sbml objects
		self.mathOnly = math_only
		if not self.mathOnly:
			SbmlObject.__init__(self, model)

	def readSbml(self, sbml_list_of_events,
					sbmlLevel=Settings.defaultSbmlLevel,
					sbmlVersion=Settings.defaultSbmlVersion):
		""" Reads the list of events from a sbml model """

		for sbml_event in sbml_list_of_events:
			event = Event(self.__model, self, self.nextId())
			event.readSbml(sbml_event, sbmlLevel, sbmlVersion)
			ListOf.add(self, event)

		SbmlObject.readSbml(self, sbml_list_of_events, sbmlLevel, sbmlVersion)

	def writeSbml(self, sbmlModel,
					sbmlLevel=Settings.defaultSbmlLevel,
					sbmlVersion=Settings.defaultSbmlVersion):
		""" Writes the list of events to a sbml model """

		for event in self:
			event.writeSbml(sbmlModel, sbmlLevel, sbmlVersion)

		if len(self) > 0:
			SbmlObject.writeSbml(self, sbmlModel.getListOfEvents(), sbmlLevel, sbmlVersion)


	def new(self, name=None):
		event = Event(self.__model, self, self.nextId())
		event.new(name)
		ListOf.add(self, event)
		# SbmlObject.new(event)
		return event


	def copy(self, obj, deletions=[], sids_subs={}, symbols_subs={}, conversion_factors={}, time_conversion=None):

		if obj not in deletions:

			SbmlObject.copy(self, obj)

			for event in obj:
				if event not in deletions:

					t_event = Event(self.__model, self, self.nextId())
					t_event.copy(
						event,
						deletions=deletions,
						sids_subs=sids_subs,
						symbols_subs=symbols_subs,
						conversion_factors=conversion_factors,
						time_conversion=time_conversion
					)
					ListOf.add(self, t_event)

	def copySubmodel(self, obj):

		for event in obj:
			t_event = Event(self.__model, self, event.objId, math_only=self.mathOnly)
			t_event.copySubmodel(event)
			ListOf.add(self, t_event)

	def withDelay(self):
		return [obj.objId for obj in self.validEvents() if obj.delay is not None]

	def nbRoots(self):

		res = 0
		for event in self.validEvents():
			res += event.trigger.nbRoots()
		return res

	def getRootsOperators(self):
		res = []
		for event in self.validEvents():
			res += event.trigger.getRootsOperator()
		return res

	def renameSbmlId(self, old_sbml_id, new_sbml_id):

		for obj in self:
			obj.renameSbmlId(old_sbml_id, new_sbml_id)

	# Events can have ids, but they are optionals !
	def sbmlIds(self):
		""" Return a set of import ids of the sbml objects """
		return [obj.getSbmlId() for obj in self if obj.getSbmlId() is not None]

	def getBySbmlId(self, sbml_id, pos=0):
		""" Find sbml objects by their import Id """

		res = []
		for obj in self:
			if obj.getSbmlId() is not None and obj.getSbmlId() == sbml_id:
				res.append(obj)

		if len(res) > 0:
			return res[pos]
		else:
			return None

	def containsSbmlId(self, sbml_id):
		""" Test if an sbml id is in the list """

		res = False
		for obj in self:
			if obj.getSbmlId() is not None and sbml_id == obj.getSbmlId():
				res = True

		return res

	def validEvents(self):

		return [
			event
			for event in self
			if event.isValid()
		]

	def nbValidEvents(self):

		res = 0
		for event in self:
			if event.isValid():
				res += 1
		return res

	def resolveXPath(self, selector):

		if not (selector.startswith("event") or selector.startswith("sbml:event")):
			raise InvalidXPath(selector)

		res_match = match(r'(.*)\[@(.*)=(.*)\]', selector)
		if res_match is None:
			raise InvalidXPath(selector)

		tokens = res_match.groups()
		if len(tokens) != 3:
			raise InvalidXPath(selector)

		object = None
		if tokens[1] == "id":
			object = self.getBySbmlId(tokens[2][1:-1])
		elif tokens[1] == "name":
			object = self.getByName(tokens[2][1:-1])
		elif tokens[1] == "metaid":
			object = self.getByMetaId(tokens[2][1:-1])

		if object is not None:
			return object

		# If not returned yet
		raise InvalidXPath(selector)

	def getByXPath(self, xpath):
		return self.resolveXPath(xpath[0]).getByXPath(xpath[1:])

	def setByXPath(self, xpath, object):
		self.resolveXPath(xpath[0]).setByXPath(xpath[1:], object)

	def getXPath(self):
		return "/".join([self.getParentObj().getXPath(), "sbml:listOfEvents"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import division

from libsignetsim.simulation.Simulation import Simulation
from libsignetsim.figure.SigNetSimFigure import SigNetSimFigure
from libsignetsim.model.math.sympy_shortcuts import SympySymbol
from libsignetsim.settings.Settings import Settings
from matplotlib.pyplot import show
from numpy import amin, amax, linspace, logspace
from os.path import join, isfile
from threading import Thread


class TimeseriesSimulation(Simulation):

	DEFAULT_NB_SAMPLES = 500

	def __init__ (self, list_of_models=[],
					experiment=None,
					time_min=Settings.simulationTimeMin,
					time_max=Settings.simulationTimeMax,
					abs_tol=Settings.defaultAbsTol,
					rel_tol=Settings.defaultRelTol,
					log_scale=Settings.simulationLogScale,
					time_ech=Settings.simulationTimeEch,
					nb_samples=Settings.simulationNbSamples,
					list_samples=None,
					keep_files=Settings.simulationKeepFiles):

		self.listOfSamples = list_samples
		if self.listOfSamples is None:
			self.buildListSamples(time_min, time_max, log_scale, time_ech, nb_samples)

		Simulation.__init__(self,
							list_of_models=list_of_models,
							time_min=Settings.simulationTimeMin,
							list_samples=self.listOfSamples,
							experiment=experiment,
							abs_tol=abs_tol,
							rel_tol=rel_tol,
							keep_files=keep_files)

		self.keepFiles = keep_files

	def buildListSamples(self, time_min, time_max, log_scale, time_ech, nb_samples):

		if time_ech is not None:
			nb_samples = int(round((time_max-time_min) / time_ech))+1

		if log_scale:
			self.listOfSamples = [float(value) for value in logspace(time_min, time_max, nb_samples)]
		else:
			self.listOfSamples = [float(value) for value in linspace(time_min, time_max, nb_samples)]

	def run(self, timeout=None):

		self.writeSimulationFiles()
		self.runSimulation(timeout=timeout)
		self.loadSimulationResults()

		if not self.keepFiles:
			self.cleanTempDirectory()

	def run_inside_thread(self, success, failure, timeout=None):
		try:
			self.run(timeout=timeout)
			success()
		except Exception as e:
			failure(e)


	def run_async(self, success, failure, timeout=None):

		t = Thread(group=None, target=self.run_inside_thread, args=(success, failure, timeout))
		t.setDaemon(True)
		t.start()

	def loadSimulationResults(self):
		self.rawData = []
		t_filename = self.getTempDirectory() + Settings.C_simulationResultsDirectory + "results_0"
		ind = 0
		while(isfile(t_filename + ("_%d" % ind))):
			(t, y) = self.readResultFile(t_filename + ("_%d" % ind))
			self.rawData.append((t, y))
			ind += 1


	def readResultFile(self, filename):

		resultsFile = open(filename, 'r')

		t = []
		trajs = {}

		variables = resultsFile.readline().strip().split(',')
		variables = [variable.strip() for variable in variables]

		for variable in variables:
			if variable != 'time':
				trajs.update({variable: []})

		for line in resultsFile.readlines():

			data = line.split()
			t.append(float(data[0]))

			for i_variable, variable in enumerate(variables):
				if variable != 'time':
					trajs[variable].append(float(data[i_variable]))

		resultsFile.close()

		# The simulations only deals with amounts, but some species are
		# Concentrations. So we need to transform them back

		for variable in self.listOfModels[0].listOfVariables:
			# print "%s : %s" % (variable.getSbmlId(), variable.symbol.getSymbol())
			if variable.isConcentration():
				t_traj = trajs[variable.getSymbolStr()]

				t_comp_traj = trajs[variable.getCompartment().getSymbolStr()]
				res_traj = []

				for i, point in enumerate(t_traj):

					if i == 0 or (i > 0 and t[i] > t[i-1]):
						if abs(t_comp_traj[i]) < self.absTol[0]:
							res_traj.append(0.0)
						else:
							res_traj.append(point / t_comp_traj[i])

				trajs.update({variable.getSymbolStr(): res_traj})
			else:
				res_traj = []

				for i, point in enumerate(trajs[variable.getSymbolStr()]):
					if i == 0 or (i > 0 and t[i] > t[i - 1]):
						res_traj.append(point)

				trajs.update({variable.getSymbolStr(): res_traj})

		t_t = []
		for i, point in enumerate(t):

			if i == 0 or (i > 0 and t[i] > t[i - 1]):
				t_t.append(point)

		t = t_t

		return (t, trajs)

	def plot(self, figure=None, plot=None, variables=[], suffix=""):


		if plot is None and figure is None:
			if self.listOfModels[0].timeUnits is not None and self.listOfModels[0].extentUnits is not None:
				figure = SigNetSimFigure(
						x_unit=self.listOfModels[0].timeUnits.getNameOrSbmlId(),
						y_unit=self.listOfModels[0].extentUnits.getNameOrSbmlId())
			else:
				figure = SigNetSimFigure()

		if plot is None and figure is not None:
			plot = figure.add_subplot(1, 1, 1)

		t, trajs = self.getRawData()[0]

		t_trajs = {}
		x_min = amin(t)
		x_max = amax(t)
		y_min = 0
		y_max = 0
		for t_id in list(trajs.keys()):

			if len(variables) == 0 or self.listOfModels[0].listOfVariables.getBySymbolStr(t_id).getSbmlId() in variables:

				y_min = min(y_min, amin(trajs[str(t_id)]))
				y_max = max(y_max, amax(trajs[str(t_id)]))
				t_trajs.update({t_id: trajs[str(t_id)]})

		for i_species, name in enumerate(t_trajs.keys()):

				t_var = self.listOfModels[0].listOfVariables.getBySymbol(SympySymbol(name))
				if not t_var.isConstant():
					figure.plot(plot, i_species, t, t_trajs[name], y_name=t_var.getNameOrSbmlId()+suffix)
					plot.legend(loc='upper right')

		plot.set_xlim([x_min, x_max])
		plot.set_ylim([y_min, y_max*1.1])
		show()

		return figure


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.AbstractTask import AbstractTask
from libsignetsim.sedml.OneStep import OneStep
from libsignetsim.sedml.SedmlException import SedmlOneStepTaskException
from libsignetsim.settings.Settings import Settings

class Task(AbstractTask):

	def __init__(self, document):

		AbstractTask.__init__(self, document)

		self.__document = document
		self.__modelReference = None
		self.__simulationReference = None

		self.__simulationObject = None
		self.__results = None

	def readSedml(self, task, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		AbstractTask.readSedml(self, task, level, version)

		if task.isSetModelReference():
			self.__modelReference = task.getModelReference()

		if task.isSetSimulationReference():
			self.__simulationReference = task.getSimulationReference()

	def writeSedml(self, task, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		AbstractTask.writeSedml(self, task, level, version)

		if self.__modelReference is not None:
			task.setModelReference(self.__modelReference)

		if self.__simulationReference is not None:
			task.setSimulationReference(self.__simulationReference)

	def getModelReference(self):
		return self.__modelReference

	def getSimulationReference(self):
		return self.__simulationReference

	def getModel(self):
		return self.__document.listOfModels.getSbmlModelByReference(self.__modelReference)

	def getSimulation(self):
		return self.__document.listOfSimulations.getSimulation(self.__simulationReference)

	def setModelReference(self, model_reference):
		self.__modelReference = model_reference

	def setSimulationReference(self, simulation_reference):
		self.__simulationReference = simulation_reference

	def setModel(self, model):
		self.__modelReference = model.getId()

	def setSimulation(self, simulation):
		self.__simulationReference = simulation.getId()

	def run(self, timeout=None):

		# One step simulations cannot be executed as a single task, they must be part of a repeated task
		# At least, that's what I understand
		if isinstance(self.__document.listOfSimulations.getSimulation(self.__simulationReference), OneStep):
			raise SedmlOneStepTaskException("One step simulations cannot be executed as a single task")

		model = self.__document.listOfModels.getSbmlModelByReference(self.__modelReference)
		simulation = self.__document.listOfSimulations.getSimulation(self.__simulationReference)
		self.__simulationObject = simulation.run(model, timeout=timeout)
		self.__results = self.__simulationObject.getRawData()[0]

	def getSimulationObject(self):
		return self.__simulationObject

	def getResults(self):
		return self.__results

	def getResultsByVariable(self, variable_sbmlid):
		return self.__results[1][variable_sbmlid]

	def getTimes(self):
		return self.__results[0]

	def getDuration(self):
		return self.__simulationObject.getSimulationDuration()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

    This file ...

"""
from __future__ import division


from libsignetsim.settings.Settings import Settings
import matplotlib.pyplot as plt


class SigNetSimFigure(object):


    default_width = 1000
    default_dpi = 600

    def __init__(self, width=default_width, dpi=default_dpi):

        self.__figure = plt.figure(figsize=(10, 6))

        self.curveIds = {}
        self.plots = []
        if dpi is None:
            self.__dpi = self.default_dpi
        else:
            self.__dpi = dpi
        if width is None:
            self.w = self.default_width/float(self.__dpi)
            self.h = (self.default_width * 0.667)/float(self.__dpi)
        else:
            self.w = width/float(self.__dpi)
            self.h = (width * 0.667)/float(self.__dpi)


    def add_subplot(self, nb_cols=1, nb_rows=None, id_cell=None, x_label=None, y_label=None):

        if nb_rows == None and id_cell == None:
            t_subplot = self.__figure.add_subplot(nb_cols,1,1)
        else:
            t_subplot = self.__figure.add_subplot(nb_cols, nb_rows, id_cell)


        self.curveIds.update({t_subplot: 0})
        # t_subplot.spines["top"].set_visible(False)
        # t_subplot.spines["right"].set_visible(False)
        # t_subplot.spines["bottom"].set_linewidth(0.3 * self.w)
        # t_subplot.spines["left"].set_linewidth(0.3 * self.w)

        t_subplot.tick_params(
            axis='x',          # changes apply to the x-axis
            which='both',      # both major and minor ticks are affected
            bottom=False,      # ticks along the bottom edge are off
            top=False,         # ticks along the top edge are off
            labelbottom=True,  # labels along the bottom edge are off
        )

        t_subplot.tick_params(
            axis='y',          # changes apply to the x-axis
            which='both',      # both major and minor ticks are affected
            left=False,        # ticks along the bottom edge are off
            right=False,       # ticks along the top edge are off
            labelbottom=True,  # labels along the bottom edge are off
        )

        if x_label is not None:
            t_subplot.set_xlabel(x_label, fontsize=int(15 * self.w))

        if y_label is not None:
            t_subplot.set_ylabel(y_label, fontsize=int(15 * self.w))

        self.plots.append(t_subplot)
        return t_subplot



    def plot(self, subplot, curve_id, x, y, x_name="", y_name=None, marker="-"):

        curve_id = self.curveIds[subplot]

        if y_name is None:
            subplot.plot(x, y, marker,
                color=Settings.color_scheme[curve_id % len(Settings.color_scheme)],
                # linewidth=int(5 * self.w),
                # label=y_name
            )
        else:
            subplot.plot(x, y, marker,
                 color=Settings.color_scheme[curve_id % len(Settings.color_scheme)],
                 # linewidth=int(5 * self.w),
                 label=y_name
            )

        self.curveIds.update({subplot: curve_id+1})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import absolute_import

from libsignetsim.model.math.sympy_shortcuts import  (
	SympySymbol, SympyInteger, SympyFloat, SympyRational, SympyAtom,
	SympyOne, SympyNegOne, SympyZero, SympyPi, SympyE, SympyExp1, SympyHalf,
	SympyInf, SympyNan, SympyAdd, SympyMul, SympyPow,
	SympyFunction, SympyUndefinedFunction, SympyLambda, SympyDerivative,
	SympyCeiling, SympyFloor, SympyAbs, SympyLog, SympyExp, SympyPiecewise,
	SympyFactorial, SympyRoot, SympyAcos, SympyAsin, SympyAtan, SympyAcosh,
	SympyAsinh, SympyAtanh, SympyCos, SympySin, SympyTan, SympyAcot,
	SympyAcoth, SympyCosh, SympySinh, SympyTanh, SympySec, SympyCsc,
	SympyCot, SympyCoth, SympyAcsc, SympyAsec,
	SympyEqual, SympyUnequal, SympyGreaterThan, SympyLessThan,
	SympyStrictGreaterThan, SympyStrictLessThan,
	SympyAnd, SympyOr, SympyXor, SympyNot, SympyTrue, SympyFalse,
	SympyMax, SympyMin)
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.SbmlMathReader import SbmlMathReader
from libsignetsim.cwriter.CMathWriter import CMathWriter
from libsignetsim.model.math.SbmlMathWriter import SbmlMathWriter
from libsignetsim.model.math.MathDevelopper import MathDevelopper
from libsignetsim.model.ModelException import SbmlException

from libsbml import parseL3Formula, formulaToL3String
from sympy import simplify, srepr

from .MathDevelopper import unevaluatedSubs

class MathFormula(SbmlMathReader, CMathWriter, SbmlMathWriter, MathDevelopper):
	""" Class for handling math formulaes """

	MATH_ERR            =  -1

	MATH_SBML           =   0
	MATH_INTERNAL       =   1
	MATH_DEVINTERNAL    =   2
	MATH_C              =   3
	MATH_PRETTYPRINT    =   4
	MATH_FINALINTERNAL  =   5

	MATH_FORMULA        =  20
	MATH_EQUATION       =  21
	MATH_VARIABLE       =  22
	MATH_KINETICLAW     =  23
	MATH_FUNCTION       =  24
	MATH_RATERULE       =  25
	MATH_EVENTASSIGNMENT=  26
	MATH_ASSIGNMENTRULE =  27
	MATH_ALGEBRAICRULE  =  28
	MATH_VALUE          =  29
	MATH_RATIONAL       =  30
	MATH_ZERO           =  31
	MATH_DELAY			=  32
	MATH_PRIORITY		=  33

	ZERO                = SympyInteger(0)
	ONE                 = SympyInteger(1)

	t                   = SympySymbol("t")

	def __init__(self, model, typeOfFormula=MATH_FORMULA, isFromReaction=None):
		""" Constructor """

		self.__model = model
		SbmlMathReader.__init__(self, model)
		CMathWriter.__init__(self, model)
		SbmlMathWriter.__init__(self, model)
		MathDevelopper.__init__(self, model)

		self.typeOfFormula = typeOfFormula

		self.sbmlLevel = model.sbmlLevel
		self.sbmlVersion = model.sbmlVersion

		self.internalTree = None
		self.simplifiedInternalTree = None
		self.simplifiedInternalTree_v2 = None

		self.isFormula = (typeOfFormula == self.MATH_FORMULA)
		self.isEquation = (typeOfFormula == self.MATH_EQUATION)
		self.isVariable = (typeOfFormula == self.MATH_VARIABLE)
		self.isKineticLaw = (typeOfFormula == self.MATH_KINETICLAW)
		self.isFunctionDefinition = (typeOfFormula == self.MATH_FUNCTION)
		self.isRateRule = (typeOfFormula == self.MATH_RATERULE)
		self.isEventAssignment = (typeOfFormula == self.MATH_EVENTASSIGNMENT)
		self.isAssignmentRule = (typeOfFormula == self.MATH_ASSIGNMENTRULE)
		self.isAlgebraicRule = (typeOfFormula == self.MATH_ALGEBRAICRULE)
		self.isDelay = (typeOfFormula == self.MATH_DELAY)
		self.isPriority = (typeOfFormula == self.MATH_PRIORITY)

		if isFromReaction is not None:
			self.isFromReaction = isFromReaction.objId
		else:
			self.isFromReaction = None

		if self.isKineticLaw:
			self.internalTree = self.ONE

		if typeOfFormula == self.MATH_ZERO:
			self.internalTree = self.ZERO

	def getSbmlMathFormula(self, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		return MathFormula.getMathFormula(self, MathFormula.MATH_SBML, sbml_level, sbml_version)


	def getPrettyPrintMathFormula(self, rawFormula=False):

		if not rawFormula:
			t_subs_mask = {}
			for t_var in self.__model.listOfSpecies:
				if t_var.isConcentration():
					t_subs_mask.update({SympySymbol(
						"_speciesForcedConcentration_%s_" % str(t_var.symbol.getInternalMathFormula())):t_var.symbol.getInternalMathFormula()})
			return formulaToL3String(self.translateForSbml(simplify(unevaluatedSubs(MathFormula.getInternalMathFormula(self), t_subs_mask))))
		else:
			return formulaToL3String(self.translateForSbml(simplify(MathFormula.getDeveloppedInternalMathFormula(self))))


	def getInternalMathFormula(self, rawFormula=True):

		if not rawFormula:
			t_subs_mask = {}
			for t_var in self.__model.listOfSpecies:
				if t_var.isConcentration():
					t_subs_mask.update({SympySymbol(
						"_speciesForcedConcentration_%s_" % str(
							t_var.symbol.getInternalMathFormula())): t_var.symbol.getInternalMathFormula(rawFormula=rawFormula)})
			return simplify(unevaluatedSubs(MathFormula.getInternalMathFormula(self), t_subs_mask))
		else:
			return MathFormula.getMathFormula(self, MathFormula.MATH_INTERNAL)


	def getDeveloppedInternalMathFormula(self, rawFormula=True):
		if not rawFormula:
			return self.translateForDeveloppedInternal(self.getInternalMathFormula(rawFormula=rawFormula))
		else:
			return MathFormula.getMathFormula(self, MathFormula.MATH_DEVINTERNAL)

	def getCMathFormula(self):
		return MathFormula.getMathFormula(self, MathFormula.MATH_C)


	def getValueMathFormula(self):
		return float(MathFormula.getMathFormula(self, MathFormula.MATH_INTERNAL))


	def getMathFormula(self,
						math_type=MATH_DEVINTERNAL,
						sbml_level=Settings.defaultSbmlLevel,
						sbml_version=Settings.defaultSbmlVersion
	):

		if math_type == self.MATH_SBML:
			return self.writeSbml(sbml_level, sbml_version)

		elif math_type == self.MATH_INTERNAL:
			return self.internalTree

		elif math_type == self.MATH_PRETTYPRINT:
			pass

		elif math_type == self.MATH_DEVINTERNAL:
			return self.translateForDeveloppedInternal(MathFormula.getMathFormula(self, self.MATH_INTERNAL))

		elif math_type == self.MATH_C:
			if MathFormula.getInternalMathFormula(self) is None:
				return "RCONST(0.0)"
			else:
				t_formula = MathFormula.getMathFormula(self, self.MATH_DEVINTERNAL)
				return self.writeCCode(t_formula)

		else:
			return "Unknown math type !"

	def setSbmlMathFormula(self, tree, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		self.setMathFormula(tree, self.MATH_SBML, sbml_level, sbml_version)

	def setInternalMathFormula(self, tree):
		self.setMathFormula(tree, self.MATH_INTERNAL)

	def setValueMathFormula(self, value):
		if isinstance(value, int):
			self.internalTree = SympyInteger(value)
		elif isinstance(value, float):
			self.internalTree = SympyFloat(value)
		else:
			self.internalTree = SympySymbol(value)

	# def setFinalMathFormula(self, tree):
	# 	self.setMathFormula(tree, math_type=self.MATH_FINALINTERNAL)

	def setPrettyPrintMathFormula(self, string, rawFormula=False):
		""" Sets the formula for substance. If not rawFormula, the species are replaced by their concentration"""

		sbml_formula = parseL3Formula(str(string))
		if sbml_formula is None:
			raise SbmlException("MathFormula : Unable to parse math formula")

		self.readSbml(sbml_formula, self.sbmlLevel, self.sbmlVersion)
		if not rawFormula:
			t_subs_mask = {}
			for t_var in self.__model.listOfSpecies:
				if t_var.isConcentration():
					t_subs_mask.update({t_var.symbol.getInternalMathFormula():SympySymbol("_speciesForcedConcentration_%s_" % str(t_var.symbol.getInternalMathFormula()))})

			self.setInternalMathFormula(self.getInternalMathFormula().subs(t_subs_mask))


	def setMathFormula(self, tree, math_type=MATH_INTERNAL, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		if math_type == self.MATH_INTERNAL:
			self.internalTree = tree
		elif math_type == self.MATH_FINALINTERNAL:
			self.internalTree = self.translateFinalForInternal(tree)


		elif math_type == self.MATH_SBML:
			self.internalTree = None
			self.readSbml(tree, sbml_level, sbml_version)


	def getMathFormulaDerivative(self, variable, mathType):
		if (mathType == self.MATH_DEVINTERNAL or mathType == self.MATH_INTERNAL):
			return self.generateDeveloppedInternalDerivative(self.getDeveloppedInternalMathFormula(), variable)

		elif mathType == self.MATH_C:
			t_derivative = self.generateDeveloppedInternalDerivative(self.getDeveloppedInternalMathFormula(), variable)
			return self.writeCCode(t_derivative)

	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		if self.getInternalMathFormula() is not None:
			old_symbol = SympySymbol(old_sbml_id)
			old_symbol_concentration = SympySymbol("_speciesForcedConcentration_%s_" % old_sbml_id)


			if old_symbol in self.getInternalMathFormula().atoms():
				self.setInternalMathFormula(unevaluatedSubs(
					self.internalTree,
					{old_symbol: SympySymbol(new_sbml_id)}
				))

			if old_symbol_concentration in self.getInternalMathFormula().atoms():
				self.setInternalMathFormula(unevaluatedSubs(
					self.internalTree,
					{old_symbol_concentration: SympySymbol("_speciesForcedConcentration_%s_" % new_sbml_id)}
				))


	def subs(self, substitutions):
		self.internalTree = self.internalTree.subs(substitutions)

	def simpleSubsDevelopped(self, substitutions):
		# print ">> simple subs : " + str(self.getDeveloppedInternalMathFormula())
		return unevaluatedSubs(self.getDeveloppedInternalMathFormula(), substitutions)


	def isOne(self):

		return (self.getMathFormula(self.MATH_DEVINTERNAL) == SympyOne
				or self.getMathFormula(self.MATH_DEVINTERNAL) == SympyInteger(1)
				or self.getMathFormula(self.MATH_DEVINTERNAL) == SympyFloat(1.0))

	def isEqual(self, string):

		return self.getInternalMathFormula() == SbmlMathReader.readSbml(self, parseL3Formula(str(string)), self.sbmlLevel, self.sbmlVersion)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.SubTask import SubTask
from libsignetsim.sedml.UniformTimeCourse import UniformTimeCourse
from libsignetsim.sedml.OneStep import OneStep
from libsignetsim.sedml.SteadyState import SteadyState
from libsignetsim.settings.Settings import Settings


class ListOfSubTasks(ListOf):

	def __init__(self, document):

		ListOf.__init__(self, document)
		self.__document = document
		self.__subTaskCounter = 0

	def new(self):

		sub_task = SubTask(self.__document)
		ListOf.append(self, sub_task)
		self.__subTaskCounter += 1
		return sub_task

	def createSubTask(self):
		return self.new()

	def readSedml(self, list_of_sub_tasks, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_sub_tasks, level, version)

		for sub_task in list_of_sub_tasks:
			t_sub_task = SubTask(self.__document)
			t_sub_task.readSedml(sub_task, level, version)
			ListOf.append(self, t_sub_task)
			self.__subTaskCounter += 1

	def writeSedml(self, list_of_sub_tasks, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_sub_tasks, level, version)

		for sub_task in self:
			t_sub_task = list_of_sub_tasks.createSubTask()
			sub_task.writeSedml(t_sub_task, level, version)

	def hasSingleTypeOfTask(self):
		""" Return true if all the subtasks are of the same type. Mixed subtasks are not implemented yet """

		types = []
		for sub_task in self:
			types.append(type(sub_task.getTask().getSimulation()))

		return len(set(types)) <= 1

	def hasSingleModel(self):

		models = []
		for sub_task in self:
			models.append(sub_task.getModel())

		return len(set(models)) <= 1

	def hasOneSteps(self):

		for sub_task in self:
			if not isinstance(sub_task.getTask().getSimulation(), OneStep):
				return False

		return True

	def hasSteadyStates(self):

		for sub_task in self:
			if not isinstance(sub_task.getTask().getSimulation(), SteadyState):
				return False

		return True

	def hasUniformTimeCourses(self):

		for sub_task in self:
			if not isinstance(sub_task.getTask().getSimulation(), UniformTimeCourse):
				return False

		return True

	def getModels(self):

		models = []
		for sub_task in self:
			models.append(sub_task.getTask().getModel())

		return list(set(models))

	def getAbsTols(self):
		abs_tols = []
		for sub_task in self:
			abs_tols.append(sub_task.getTask().getSimulation().getAlgorithm().getAbsTol())

		return abs_tols

	def getRelTols(self):
		rel_tols = []
		for sub_task in self:
			rel_tols.append(sub_task.getTask().getSimulation().getAlgorithm().getRelTol())

		return rel_tols


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Testing the research for conservation laws, and the subsequent reduction

"""
from libsignetsim import Model, KineticLaw, TimeseriesSimulation

from unittest import TestCase


class TestReduceModelMultiCompartment(TestCase):
	""" Tests high level functions """


	def testReduceModel(self):

		model = Model()
		cytosol = model.listOfCompartments.new("Cytosol", value=100)
		membrane = model.listOfCompartments.new("Membrane", value=10)

		ras_cytosol = model.listOfSpecies.new("Ras (cytosol)", cytosol, 27.5)
		ras_membrane = model.listOfSpecies.new("Ras (membrane)", membrane, 0)

		kf = model.listOfParameters.new("To cytosol", 1)
		kr = model.listOfParameters.new("To membrane", 1)

		r = model.listOfReactions.new("Ras transport")
		r.listOfReactants.add(ras_membrane)
		r.listOfProducts.add(ras_cytosol)

		r.setKineticLaw(KineticLaw.MASS_ACTION, reversible=True, parameters=[kf, kr])

		model.build(reduce=False)

		sim = TimeseriesSimulation([model], time_min=0, time_max=100, time_ech=10)
		sim.run()
		t, ys = sim.getRawData()[0]

		model.build(reduce=True, vars_to_keep=[ras_cytosol.getSymbolStr()])

		sim_reduced = TimeseriesSimulation([model], time_min=0, time_max=100, time_ech=10)
		sim_reduced.run()
		t_reduced, ys_reduced = sim_reduced.getRawData()[0]

		for i, y in enumerate(ys['Ras_cytosol']):
			self.assertAlmostEqual(y, ys_reduced['Ras_cytosol'][i], delta=y*1e-4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import division

from libsignetsim.model.sbml.EventAssignedVariable import EventAssignedVariable
from libsignetsim.model.sbml.InitiallyAssignedVariable import InitiallyAssignedVariable
from libsignetsim.model.sbml.RuledVariable import RuledVariable
from libsignetsim.model.Variable import Variable
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasUnits import HasUnits
from libsignetsim.model.sbml.HasConversionFactor import HasConversionFactor
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.sympy_shortcuts import SympyFloat
from libsignetsim.model.ModelException import InvalidXPath


class Species(SbmlObject, Variable, InitiallyAssignedVariable,
						RuledVariable, EventAssignedVariable, HasUnits, HasConversionFactor, HasParentObj):

	def __init__(self, model, parent_obj, objId):

		self.__model = model
		self.objId = objId

		HasParentObj.__init__(self, parent_obj)
		Variable.__init__(self, model, Variable.SPECIES)
		SbmlObject.__init__(self, model)
		InitiallyAssignedVariable.__init__(self, model)
		RuledVariable.__init__(self, model)
		EventAssignedVariable.__init__(self, model)
		HasUnits.__init__(self, model)
		HasConversionFactor.__init__(self, model)

		self.__compartment = None
		self.boundaryCondition = False
		self.hasOnlySubstanceUnits = False
		self.isDeclaredConcentration = True
		self.concentrationUnit = None

	def new(self, name=None, compartment=None, value=0, unit=None,
			 constant=False, boundaryCondition=False, hasOnlySubstanceUnits=False):


		if compartment != None:
			self.__compartment = compartment.getSbmlId()

		else:
			if len(self.__model.listOfCompartments) == 0:
				self.__model.listOfCompartments.new("cell")

			self.__compartment = self.__model.listOfCompartments[0].getSbmlId()

		SbmlObject.new(self)
		Variable.new(self, name, Variable.SPECIES)
		HasUnits.new(self, unit)

		self.setValue(value)

		self.constant = constant
		self.boundaryCondition = boundaryCondition
		self.hasOnlySubstanceUnits = hasOnlySubstanceUnits
		self.setUnits(self.__model.getSubstanceUnits())

	def copy(self, obj, sids_subs={}, symbols_subs={}, usids_subs={}, conversion_factor=None):

		self.setCompartment(obj.getCompartment(), sids_subs=sids_subs)

		Variable.copy(self, obj, sids_subs=sids_subs, symbols_subs=symbols_subs, conversion_factor=conversion_factor)
		SbmlObject.copy(self, obj)
		HasUnits.copy(self, obj, usids_subs=usids_subs)
		HasConversionFactor.copy(self, obj)

		self.constant = obj.constant
		self.boundaryCondition = obj.boundaryCondition
		self.hasOnlySubstanceUnits = obj.hasOnlySubstanceUnits
		self.isDeclaredConcentration = obj.isDeclaredConcentration

	def readSbml(self, sbml_species, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		if sbml_level >= 2:
			self.__compartment = sbml_species.getCompartment()
		else:
			self.__compartment = sbml_species.getCompartment()

		Variable.readSbml(self, sbml_species, sbml_level, sbml_version)
		SbmlObject.readSbml(self, sbml_species, sbml_level, sbml_version)
		HasUnits.readSbml(self, sbml_species, sbml_level, sbml_version)

		if HasUnits.getUnits(self) is None and self.__model.substanceUnits is not None:
			HasUnits.setUnits(self, self.__model.substanceUnits)


	def readSbmlVariable(self, sbml_species, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		self.symbol.readSbml(sbml_species.getId(), sbml_level, sbml_version)

		if sbml_species.isSetHasOnlySubstanceUnits():
			self.hasOnlySubstanceUnits = sbml_species.getHasOnlySubstanceUnits()

		elif sbml_level in [1,2]:
			self.hasOnlySubstanceUnits = False

		if sbml_species.isSetInitialAmount():
			self.isInitialized = True
			self.isDeclaredConcentration = False
			self.value.readSbml(sbml_species.getInitialAmount(), sbml_level, sbml_version)

		elif sbml_species.isSetInitialConcentration():
			self.isInitialized = True
			self.isDeclaredConcentration = True
			self.value.readSbml(sbml_species.getInitialConcentration(), sbml_level, sbml_version)
			self.value.setInternalMathFormula(self.value.getInternalMathFormula()*self.getCompartment().symbol.getInternalMathFormula())

		if sbml_species.isSetBoundaryCondition():
			self.boundaryCondition = sbml_species.getBoundaryCondition()

		elif sbml_level in [1,2]:
			self.boundaryCondition = False

		if sbml_species.isSetConstant():
			self.constant = sbml_species.getConstant()

		elif sbml_level in [1,2]:
			self.constant = False

		if sbml_level >= 3 and sbml_species.isSetConversionFactor():
			HasConversionFactor.readSbml(self, sbml_species.getConversionFactor(), sbml_level, sbml_version)

	def writeSbml(self, sbml_model, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		sbml_sp = sbml_model.createSpecies()

		Variable.writeSbml(self, sbml_sp, sbml_level, sbml_version)
		SbmlObject.writeSbml(self, sbml_sp, sbml_level, sbml_version)
		HasUnits.writeSbml(self, sbml_sp, sbml_level, sbml_version)

		sbml_sp.setCompartment(self.getCompartment().getSbmlId())


	def writeSbmlVariable(self, sbml_sp, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		if self.isInitialized:
			t_value = self.value.getSbmlMathFormula(sbml_level, sbml_version)
			if not self.isDeclaredConcentration:
				sbml_sp.setInitialAmount(t_value.getValue())
			else:
				t_formula = MathFormula(self.__model)
				t_formula.setInternalMathFormula(
					self.value.getInternalMathFormula()/self.getCompartment().symbol.getInternalMathFormula()
				)
				sbml_sp.setInitialConcentration(t_formula.getValueMathFormula())

		if self.boundaryCondition is not None and (sbml_level >= 3 or self.boundaryCondition is not False):
			sbml_sp.setBoundaryCondition(self.boundaryCondition)

		if ((sbml_level == 2 and self.hasOnlySubstanceUnits)
			or sbml_level == 3):
			sbml_sp.setHasOnlySubstanceUnits(self.hasOnlySubstanceUnits)

		if self.constant is not None and (sbml_level == 3 or (sbml_level == 2 and self.constant)):
			sbml_sp.setConstant(self.constant)


		if sbml_level >= 3 and self.isSetConversionFactor():
			HasConversionFactor.writeSbml(self, sbml_sp, sbml_level, sbml_version)

	def isInReactions(self, including_fast_reactions=False, including_modifiers=False):
		""" The purpose of this function is to test is the species's amount
			is actually modified by a reaction. Thus not checking the Modifiers
			makes sense.
		"""

		for i_reaction, reaction in enumerate(self.__model.listOfReactions):
			# print "\n> reaction #%d" % i_reaction
			if not reaction.fast or including_fast_reactions:
				# print "well, first, it's not fast"
				if reaction.listOfReactants:
					for i, reactant in enumerate(reaction.listOfReactants):
						# print ">>> reactant #%d : %s" % (i, reactant.getSpecies().getSbmlId())
						if reactant.getSpecies() == self:
							return True

				# # TODO
				# # This might break some stuff, needs to pass the sbml test suite
				# if reaction.listOfModifiers:
				#     for modifier in reaction.listOfModifiers.values():
				#         if modifier.getSpecies() == self:
				#             return True

				if reaction.listOfProducts:
					for i, product in enumerate(reaction.listOfProducts):
						# print ">>> product #%d : %s" % (i, product.getSpecies().getSbmlId())
						if product.getSpecies() == self:
							return True
				if including_modifiers and reaction.listOfModifiers:
					for i, modifier in enumerate(reaction.listOfModifiers):
						# print ">>> product #%d : %s" % (i, product.getSpecies().getSbmlId())
						if modifier.getSpecies() == self:
							return True
		return False

	def isInFastReactions(self):
		""" The purpose of this function is to test is the species's amount
			is actually modified by a reaction. Thus not checking the Modifiers
			makes sense.
		"""

		for reaction in self.__model.listOfReactions:
				if reaction.listOfReactants:
					for reactant in reaction.listOfReactants:
						if reactant.getSpecies() == self:
							if reaction.fast:
								return True
							else:
								return False

				if reaction.listOfProducts:
					for product in reaction.listOfProducts:
						if product.getSpecies() == self:
							if reaction.fast:
								return True
							else:
								return False

		return False

	def isOnlyInFastReactions(self):
		""" The purpose of this function is to test is the species's amount
			is actually modified by a reaction. Thus not checking the Modifiers
			makes sense.
		"""

		for reaction in self.__model.listOfReactions:
				if reaction.listOfReactants:
					for reactant in reaction.listOfReactants:
						if reactant.getSpecies() == self:
							if not reaction.fast:
								return False

				if reaction.listOfProducts:
					for product in reaction.listOfProducts:
						if product.getSpecies() == self:
							if not reaction.fast:
								return False

		return True

	def getODE(self, including_fast_reactions=True, rawFormula=False, symbols=False):

		t_formula = MathFormula(self.__model)

		if self.constant == True:
			t_formula.setInternalMathFormula(MathFormula.ZERO)
			return t_formula

		elif self.isRateRuled():
			t_rule = self.isRuledBy().getDefinition(rawFormula=rawFormula).getInternalMathFormula()

			if t_rule is None:
				t_rule = MathFormula.ZERO

			if self.hasOnlySubstanceUnits:
				t_formula.setInternalMathFormula(t_rule)

			elif self.getCompartment().isRateRuled() and not t_rule == MathFormula.ZERO:
				""" Then things get complicated. We need to add a term :
					 amount_species * rate_comp / comp
				"""
				t_amount_species = self.symbol.getInternalMathFormula()
				t_comp_rate = self.getCompartment().isRuledBy().getDefinition(rawFormula=rawFormula).getInternalMathFormula()
				t_comp = self.getCompartment().symbol.getInternalMathFormula()

				t_formula.setInternalMathFormula(t_rule + t_amount_species*t_comp_rate/t_comp)
			else:
				t_formula.setInternalMathFormula(t_rule)
			return t_formula

		elif self.isInReactions(including_fast_reactions):

			ode = MathFormula.ZERO

			if not self.boundaryCondition:
				for reaction in self.__model.listOfReactions:
					if not reaction.fast or including_fast_reactions:
						ode += reaction.getODE(self, symbols=symbols, rawFormula=rawFormula).getInternalMathFormula()

			if self.isSetConversionFactor():
				ode *= self.getSymbolConversionFactor()

			elif self.__model.isSetConversionFactor():
				ode *= self.__model.getSymbolConversionFactor()

			t_formula.setInternalMathFormula(ode)
			return t_formula



	def getMathValue(self, rawFormula=False):

		if rawFormula and (self.isDeclaredConcentration or not self.hasOnlySubstanceUnits):
			t_formula = MathFormula(self.__model)
			t_formula.setInternalMathFormula(
				self.value.getInternalMathFormula()/self.getCompartment().symbol.getInternalMathFormula()
			)
			return t_formula
		else:
			return self.value


	def getValue(self):

		if self.isInitialized:
			if self.isDeclaredConcentration:
				return float(self.value.getInternalMathFormula()/self.getCompartment().symbol.getInternalMathFormula())
			else:
				return Variable.getValue(self)


	def setValue(self, value):

		if value is None:
			self.isInitialized = False
			self.value.setInternalMathFormula(None)

		else:
			self.isInitialized = True
			if self.isDeclaredConcentration:

				self.value.setInternalMathFormula(SympyFloat(value)*self.getCompartment().symbol.getInternalMathFormula())
			else:
				Variable.setValue(self, value)


	def hasUnits(self):

		if self.hasOnlySubstanceUnits:
			return HasUnits.hasUnits(self)
		else:
			return HasUnits.hasUnits(self) and self.getCompartment().hasUnits()



	def setUnits(self, unit, prefix=""):

		if unit is not None:
			if self.hasOnlySubstanceUnits:
				HasUnits.setUnits(self, unit, prefix)
			else:
				if unit is not None and self.getCompartment().getUnits() is not None:
					HasUnits.setUnits(self, self.__model.listOfUnitDefinitions.getAmountUnit(unit,
														self.getCompartment().getUnits()), prefix)

	def getUnits(self):

		if self.hasOnlySubstanceUnits:
			return HasUnits.getUnits(self)
		else:
			if self.concentrationUnit is None:
				if HasUnits.getUnits(self) is not None and self.getCompartment().getUnits() is not None:
					self.concentrationUnit = self.__model.listOfUnitDefinitions.getConcentrationUnit(HasUnits.getUnits(self),
															self.getCompartment().getUnits())
					return self.concentrationUnit
			else:
				return self.concentrationUnit


	def getCompartment(self):
		if self.__compartment is not None:
			return self.__model.listOfCompartments.getBySbmlId(self.__compartment)

	def setCompartment(self, compartment, sids_subs={}):

		if compartment.getSbmlId() in list(sids_subs.keys()):
			self.__compartment = sids_subs[compartment.getSbmlId()]
		else:
			self.__compartment = compartment.getSbmlId()

	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		if self.__compartment == old_sbml_id:
			self.__compartment = new_sbml_id


	def getByXPath(self, xpath):

		if len(xpath) == 0:
			return self

		if len(xpath) > 1:
			return InvalidXPath("/".join(xpath))

		if xpath[0] == "@value":
			return self.getValue()

		elif xpath[0] == "@name":
			return self.getName()

		elif xpath[0] == "@id":
			return self.getSbmlId()


	def setByXPath(self, xpath, object):

		# In this case, we assume it points to the value
		if len(xpath) == 0:
			return self.setValue(object)

		if len(xpath) > 1:
			return InvalidXPath("/".join(xpath))

		if xpath[0] == "@value":
			return self.setValue(object)

		elif xpath[0] == "@name":
			return self.setName(object)

		elif xpath[0] == "@id":
			return self.setSbmlId(object)

		else:
			return InvalidXPath("/".join(xpath))

	def getXPath(self, attribute=None):

		xpath = "sbml:species"
		if self.__model.sbmlLevel == 1:
			xpath += "[@name='%s']" % self.getSbmlId()
		else:
			xpath += "[@id='%s']" % self.getSbmlId()

		if attribute is not None:
			xpath += "/@%s" % attribute

		return "/".join([self.getParentObj().getXPath(), xpath])


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function
from __future__ import division

from builtins import str
from builtins import range
from past.utils import old_div
from libsignetsim.simulation.TimeseriesSimulation import TimeseriesSimulation
from libsignetsim.model.SbmlDocument import SbmlDocument

from os.path import join, exists
from os import mkdir
import numpy

class BiomodelsTestCaseSimulation(TimeseriesSimulation):

	CASES_PATH = "libsignetsim/tests/biomodels/cases"

	def __init__ (self, model_id, time_ech=0.01, time_max=10, abs_tol=1e-12, rel_tol=1e-6, test_export=False, keep_files=True):

		self.model_id = str(model_id)
		self.testExport = test_export
		self.timeMin = 0
		self.timeEch = time_ech
		self.timeMax = time_max
		self.testAbsTol = 1e-2
		self.testRelTol = 1e-6
		self.sbmlIdToPlot = []
		self.sbmlIdToPlotAmount = []
		self.sbmlIdToPlotConcentrations = []

		self.simulatedData = None
		self.expectedData = None

		self.loadTestCaseModel()
		self.loadTestCaseResults()
		# self.testExport= True
		TimeseriesSimulation.__init__(self,
										list_of_models=[self.model],
										time_min=self.timeMin,
										time_max=self.timeMax,
										time_ech=self.timeEch,
										abs_tol=abs_tol,
										rel_tol=rel_tol,
										keep_files=keep_files)

	def run(self):

		TimeseriesSimulation.run(self)

		self.simulatedData = self.getRawData()[0]
		self.writeTestOutput()

		return self.checkResults()

	def loadTestCaseModel(self):

		document = SbmlDocument()
		document.readSbmlFromFile(self.getModelFilename())
		self.model = document.getModelInstance()

	def loadTestCaseResults(self):

		results = open(self.getExpectedResultsFilename(), "r")
		header = results.readline()
		results.close()

		header = header[3:len(header)-2].split(',')
		header = [head.strip() for head in header]
		header = [head[1:len(head)-1] for head in header]

		for var in header:
			if var == "time":
				pass

			else:
				self.sbmlIdToPlot.append(var)
				self.sbmlIdToPlotConcentrations.append(var)

		data = numpy.genfromtxt(self.getExpectedResultsFilename(), skip_header=1)
		self.expectedData = (data[:,0], data[:,1:data.shape[0]-1])

	def getModelFilename(self):
		return "%s/%s/model.xml" % (self.CASES_PATH, self.model_id)

	def getExpectedResultsFilename(self):
		return "%s/%s/results.csv" % (self.CASES_PATH, self.model_id)

	def getResultsFilename(self):
		return "%s/%s/results_simulated.csv" % (self.CASES_PATH, self.model_id)
	#
	# def getTemporaryModelFilename(self):
	# 	return "%s/t_%s.xml" % (self.CASES_PATH, self.model_id)

	def checkResults(self):

		DEBUG = False
		precisionError = False

		if self.expectedData is not None and self.simulatedData is not None:

			(traj_times, trajs) = self.simulatedData
			(etraj_times, etrajs) = self.expectedData


			for t in range(etrajs.shape[0]):

				if etraj_times[t] in traj_times:

					traj_pos = traj_times.index(etraj_times[t])

					for i_var in range(etrajs.shape[1]):

						t_expected_value = float(etrajs[t,i_var])
						var = self.sbmlIdToPlot[i_var]

						if self.listOfModels[0].listOfVariables.containsSbmlId(var):
							t_var = self.listOfModels[0].listOfVariables.getBySbmlId(var)

							t_value = None
							if t_var.isSpecies():
								t_compartment = t_var.getCompartment()

								# Here we ask for the concentration but it's declared an amount
								if t_var.symbol.getPrettyPrintMathFormula() in self.sbmlIdToPlotConcentrations and t_var.hasOnlySubstanceUnits:
									t_value = old_div(trajs[t_var.symbol.getPrettyPrintMathFormula()][traj_pos],trajs[t_compartment.symbol.getPrettyPrintMathFormula()][traj_pos])

								else:
									t_value = trajs[t_var.symbol.getPrettyPrintMathFormula()][traj_pos]
							else:
								t_value = trajs[t_var.symbol.getPrettyPrintMathFormula()][traj_pos]

							if abs(t_value-t_expected_value) > (self.testAbsTol + self.testRelTol*abs(t_expected_value)):
								if DEBUG:
									print("%s : %.10g, %.10g (%.0f%%, %.2g)" % (var, t_value, t_expected_value, abs(t_value-t_expected_value)/t_expected_value*100, etraj_times[t]))

								precisionError = True
						else:
							print("cannot find variable %s" % var)
			if precisionError:
				return -2
			return 0
		else:
			return -1

	def writeTestOutput(self):

		if self.rawData is not None:

			results_file = open(self.getResultsFilename(), 'w')
			(traj_times, trajs) = self.rawData[0]

			# Writing header
			line = "# ['time'"

			for sbml_id in self.sbmlIdToPlot:
				if self.listOfModels[0].listOfVariables.containsSbmlId(sbml_id):
					t_var = self.listOfModels[0].listOfVariables.getBySbmlId(sbml_id)
					if t_var.isSpecies():
						line += ",'%s'" % sbml_id

			results_file.write(line + "]\n")

			for i_t, t_time in enumerate(traj_times):
				line = "%.14f" % t_time

				for sbml_id in self.sbmlIdToPlot:
					if self.listOfModels[0].listOfVariables.containsSbmlId(sbml_id):
						t_var = self.listOfModels[0].listOfVariables.getBySbmlId(sbml_id)
						if t_var.isSpecies():

							t_compartment = t_var.getCompartment()
							# Here we ask for the concentration but it's declared an amount
							if t_var.getSbmlId() in self.sbmlIdToPlotConcentrations and t_var.hasOnlySubstanceUnits:
								line += " %.16g" % (old_div(trajs[t_var.getSbmlId()][i_t],trajs[t_compartment.getSbmlId()][i_t]))

							else:
								line += " %.16g" % trajs[t_var.getSbmlId()][i_t]

				results_file.write(line + "\n")

			results_file.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.numl.Dimension import Dimension
from libsignetsim.settings.Settings import Settings

class AtomicValue (Dimension):

	def __init__(self, document, result_component, description=None, value=None):
		Dimension.__init__(self, document, result_component, description)
		self.__document = document
		self.__value = value

	def readNuML(self, atomic_value, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		Dimension.readNuML(self, atomic_value, level, version)

		# Weird empty tag
		if atomic_value.getValue()[0] != "\n":
			self.__value = atomic_value.getValue()


	def writeNuML(self, atomic_value, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		Dimension.writeNuML(self, atomic_value, level, version)
		if self.__value is not None:
			atomic_value.setValue(str(self.__value))

	def getValue(self):
		return self.__value


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is a simple example of reading and writing a basic NuML doc

"""

from libsignetsim import NuMLDocument, Settings

from unittest import TestCase
from os.path import join, dirname


class TestExample(TestCase):
	""" Tests high level functions """

	def testExample(self):

		numl_doc = NuMLDocument()
		numl_doc.readNuMLFromFile(join(join(dirname(__file__), "files"), "example.xml"))

		# Trying to reproduce the example 5.2 from the specification,
		# but libnuml doesn't allow a tuple description as the root description,
		# seems like we need a composite description in the middle
		result = numl_doc.listOfResultComponents.createResultComponent()
		result.setId("main_fitting_result")
		main = result.createCompositeDescription(name=None, index_type="double")
		tuple = main.createTupleDescription()
		objective_value = tuple.createAtomicDescription("Objective value", "float")
		root_square_mean = tuple.createAtomicDescription("Root mean square", "float")
		standard_deviation = tuple.createAtomicDescription("Standard deviation", "float")

		main_value = result.createCompositeValue(main, index_value="0")
		tuple_value = main_value.createTupleValue(tuple)
		tuple_value.createAtomicValue(objective_value, 12.5015)
		tuple_value.createAtomicValue(root_square_mean, 0.158123)
		tuple_value.createAtomicValue(standard_deviation, 0.159242)

		numl_doc.writeNuMLToFile(join(Settings.tempDirectory, "example_copy.xml"))
		numl_doc = NuMLDocument()
		numl_doc.readNuMLFromFile(join(Settings.tempDirectory, "example_copy.xml"))
		numl_doc.writeNuMLToFile(join(Settings.tempDirectory, "example_copy_copy.xml"))








#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function
from __future__ import absolute_import
from six import string_types
# from builtins import str

from libsignetsim.sedml.math.SedmlMathReader import SedmlMathReader
from libsignetsim.sedml.math.SedmlMathWriter import SedmlMathWriter
from libsignetsim.settings.Settings import Settings
from .sympy_shortcuts import (SympySymbol, SympyInteger, SympyFloat, SympyTrue, SympyFalse, SympyLambda)

import libsbml
from libsedml import formulaToString, ASTNode, parseFormula, parseL3Formula
from six.moves import reload_module
reload_module(libsbml)

from sympy import srepr
from numpy import sum, product

class MathFormula(SedmlMathReader, SedmlMathWriter):
	""" Class for handling math formulaes """

	MATH_SBML = 0
	MATH_INTERNAL = 1
	MATH_PRETTYPRINT = 2
	MATH_VALUE = 3

	ZERO                = SympyInteger(0)
	ONE                 = SympyInteger(1)
	t                   = SympySymbol("t")

	def __init__(self, document):
		""" Constructor """

		self.__document = document
		SedmlMathReader.__init__(self, document)
		SedmlMathWriter.__init__(self, document)

		self.level = None
		self.version = None
		self.internalTree = None

	def readSedml(self, formula, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		""" Import a math formula from sedml """
		# print ">> input : %s" % formulaToString(formula)

		self.level = level
		self.version = version
		if formula is not None:
			self.internalTree = SedmlMathReader.translateForInternal(self, formula)

			if Settings.verbose >= 2:
				print("\n> readSedml : ")
				print(">> input : %s" % self.printSbml(formula))
				print(">> output simplified : %s" % str(self.internalTree))
				print(">> output : %s" % srepr(self.internalTree))

	def writeSedml(self, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		""" Export math formula to sbml """

		if self.internalTree is not None:
			formula = SedmlMathWriter.translateForSedml(self, self.internalTree, level, version)

			if Settings.verbose >= 2:
				print("\n> writeSedml")
				print(">> input : %s" % srepr(self.internalTree))
				print(">> input simplified : %s" % str(self.internalTree))
				print(">> output : %s" % self.printSbml(formula, level, version))

			return formula



	def printSbml(self, formula, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		if isinstance(formula, string_types):
			return formula
		elif isinstance(formula, ASTNode):# or isinstance(formula, ASTNodeSBML):
			return formulaToString(formula)
		else:
			return str(formula)

	def evaluateMathFormula(self, data, parameters):

		nb_timepoints = len(list(data.values())[0])
		return self.evaluate(self.internalTree, data, parameters, nb_timepoints)

	def evaluate(self, expr, data, parameters, nb_timepoints):
		# print "evaluating %s" % str(expr)

		if expr.func == SympyFloat:
			return [float(expr)]*nb_timepoints

		elif expr.func == SympyInteger:
			return [int(expr)]*nb_timepoints

		elif expr == SympyTrue:
			return [True]*nb_timepoints

		elif expr == SympyFalse:
			return [False]*nb_timepoints

		elif expr in list(data.keys()):
			return data[expr]

		elif expr in list(parameters.keys()):
			return [parameters[expr]]*nb_timepoints

		elif str(expr.func) in ["min", "max", "sum", "product"]:
			evaluated_arg = self.evaluate(expr.args[0], data, parameters, nb_timepoints)

			if str(expr.func) == "min":
				return [min(evaluated_arg)]*nb_timepoints
			elif str(expr.func) == "max":
				return [max(evaluated_arg)]*nb_timepoints
			elif str(expr.func) == "sum":
				return [sum(evaluated_arg)]*nb_timepoints
			elif str(expr.func) == "product":
				return [product(evaluated_arg)]*nb_timepoints

		else:

			evaluated_args = [self.evaluate(arg, data, parameters, nb_timepoints) for arg in expr.args]
			# print evaluated_args
			values = []
			for timepoint in range(nb_timepoints):
				evaluated_timepoint = (arg[timepoint] for arg in evaluated_args)
				# print list(evaluated_timepoint)
				values.append(expr.func(*evaluated_timepoint))

			return values


	#
	# def getSbmlMathFormula(self, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
	# 	return MathFormula.getMathFormula(self, MathFormula.MATH_SBML, sbml_level, sbml_version)
	#
	#
	def getPrettyPrint(self):
		return str(self.internalTree)
	#
	#
	def getInternalMathFormula(self):
		return MathFormula.getMathFormula(self, MathFormula.MATH_INTERNAL)
	#
	#
	# def getDeveloppedInternalMathFormula(self):
	# 	return MathFormula.getMathFormula(self, MathFormula.MATH_DEVINTERNAL)
	#
	#
	# def getValueMathFormula(self):
	# 	return float(MathFormula.getMathFormula(self, MathFormula.MATH_INTERNAL))
	#
	#
	def getMathFormula(self, math_type=MATH_INTERNAL):

		if math_type == self.MATH_SBML:
			pass
			# return self.writeSbml(sbml_level, sbml_version)

		elif math_type == self.MATH_INTERNAL:
			return self.internalTree

		elif math_type == self.MATH_PRETTYPRINT:
			pass

		else:
			return "Unknown math type !"

	def setSbmlMathFormula(self, tree, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		self.setMathFormula(tree, self.MATH_SBML, sbml_level, sbml_version)

	def setInternalMathFormula(self, tree):
		self.setMathFormula(tree, self.MATH_INTERNAL)
	#
	# def setValueMathFormula(self, value):
	# 	if isinstance(value, int):
	# 		self.internalTree = SympyInteger(value)
	# 	elif isinstance(value, float):
	# 		self.internalTree = SympyFloat(value)
	# 	else:
	# 		self.internalTree = SympySymbol(value)
	#
	# def setFinalMathFormula(self, tree):
	# 	self.setMathFormula(tree, math_type=self.MATH_FINALINTERNAL)
	#
	def setPrettyPrintMathFormula(self, string):

		try:
			sbml_formula = parseFormula(string)
			if sbml_formula is not None:
				self.readSedml(sbml_formula, self.level, self.version)
			else:
				sbml_formula = parseL3Formula(string)
				self.readSedml(sbml_formula, self.level, self.version)

		except:
			try:
				sbml_formula = parseL3Formula(string)
				self.readSedml(sbml_formula, self.level, self.version)

			except:
				raise Exception("MathFormula : Unable to parse math formula")



	def setMathFormula(self, tree, math_type=MATH_INTERNAL, sbml_level=Settings.defaultSedmlLevel, sbml_version=Settings.defaultSedmlVersion):

		if math_type == self.MATH_INTERNAL:
			self.internalTree = tree

		elif math_type == self.MATH_SBML:
			self.internalTree = None
			self.readSbml(tree, sbml_level, sbml_version)

		elif math_type == self.MATH_PRETTYPRINT:
			pass
	#
	#
	# def renameSbmlId(self, old_sbml_id, new_sbml_id):
	#
	# 	old_symbol = SympySymbol(old_sbml_id)
	# 	if old_symbol in self.getInternalMathFormula().atoms():
	# 		self.setInternalMathFormula(self.getInternalMathFormula().subs(old_symbol, SympySymbol(new_sbml_id)))
	#
	# def subs(self, substitutions):
	# 	self.internalTree = self.internalTree.subs(substitutions)
	#
	# def simpleSubs(self, substitutions):
	# 	return unevaluatedSubs(self.internalTree, substitutions)
	#
	#
	# def isOne(self):
	#
	# 	return (self.getMathFormula(self.MATH_DEVINTERNAL) == SympyOne
	# 			or self.getMathFormula(self.MATH_DEVINTERNAL) == SympyInteger(1)
	# 			or self.getMathFormula(self.MATH_DEVINTERNAL) == SympyFloat(1.0))
	#
	# def isEqual(self, string):
	#
	# 	return self.getInternalMathFormula() == SbmlMathReader.readSbml(self, parseL3Formula(str(string)), self.sbmlLevel, self.sbmlVersion)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings

class HasId(object):

	def __init__(self, document):

		self.__document = document
		self.__id = None
		self.__name = None
		self.__document.listOfIds.append(self)

	def readSedml(self, has_id_object, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		if has_id_object.isSetId():
			self.__id = has_id_object.getId()

		if has_id_object.isSetName():
			self.__name = has_id_object.getName()

	def writeSedml(self, has_id_object, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		if self.__id is not None:
			has_id_object.setId(self.__id)

		if self.__name is not None:
			has_id_object.setName(str(self.__name))

	def getId(self):
		return self.__id

	def getName(self):
		return self.__name

	def setId(self, id):
		self.__id = id

	def setName(self, name):
		self.__name = name